/* -*- Mode: JavaScript; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* Copyright (c) 2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/*
 * javascript debugger commandline handler; handles starting debug sessions
 * from the command line.
 */

const nsISupports           = Components.interfaces.nsISupports;
const nsICategoryManager    = Components.interfaces.nsICategoryManager;
const nsIComponentRegistrar = Components.interfaces.nsIComponentRegistrar;
const nsICommandLine        = Components.interfaces.nsICommandLine;
const nsICommandLineHandler = Components.interfaces.nsICommandLineHandler;
const nsIFactory            = Components.interfaces.nsIFactory;
const nsIModule             = Components.interfaces.nsIModule;
const nsIWindowWatcher      = Components.interfaces.nsIWindowWatcher;
const nsIObserver           = Components.interfaces.nsIObserver;
/*
 * Classes
 */

const JSDBGPConsoleHandler = {
    param: null,
    onload: null,
    jsshell: false,
    windowHasOpened: false,
    /* nsISupports */
    QueryInterface : function(iid) {
        if (iid.equals(nsICommandLineHandler) ||
            iid.equals(nsIObserver) ||
            iid.equals(nsIFactory) ||
            iid.equals(nsISupports))
            return this;

        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    /* nsICommandLineHandler */

    handle : function(cmdLine) {
        try {
            if (cmdLine.findFlag("jsdbgp", false) < 0) {
                return;
            }
            if (cmdLine.findFlag("jsshell", false) >= 0) {
                cmdLine.handleFlag("jsshell", false);
                this.jsshell = true;
            }
        try {
            this.param = cmdLine.handleFlagWithParam("jsdbgp", false);
            if (this.param == null) {
                return;
            }
            // we've got configuration stuff, lets configure jsdbgp
        } catch(e) {
            // we got just -casper, so we'll start the xul interface
        }
        // we need to wait for the main window to startup before we
        // run any tests
        this.windowWatcher.registerNotification(this);
        } catch(e) {
            dump(e+"\n");
        }
    },

    helpInfo : "       -jsdbgp <config>        js dbgp configuration.\n"+
               "       -jsshell                start a js interactive shell.\n",

    /* nsIFactory */

    createInstance : function(outer, iid) {
        if (outer != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;

        return this.QueryInterface(iid);
    },

    lockFactory : function(lock) {
        /* no-op */
    },

    get windowWatcher() {
        return Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                        .getService(Components.interfaces.nsIWindowWatcher);
    },
    
    observe: function(subject, topic, data)
    {
        switch(topic) {
        case "domwindowopened":
            try {
                var domWindow = subject.QueryInterface(Components.interfaces.nsIDOMWindow);
                this.windowWatcher.unregisterNotification(this);
                // now we install an event listener and wait for the load event
                var self = this;
                var handler = function(event) {
                    try {
                    domWindow.removeEventListener("load", handler, false);
                    self.handleLoad(event);
                    } catch(e) {
                        dump(e+"\n");
                    }
                }
                domWindow.addEventListener("load", handler, false);
            } catch(e) {
                dump(e+"\n");
            }
            break;
        }
    },
    
    handleLoad: function(event) {
        try {
            if (event.type != "load") return;
            // target is document, currentTarget is the chromeWindow
            // is this a xul window?
            if (event.target.contentType != 'application/vnd.mozilla.xul+xml')
                return;
            
            // XXX we need to implement this stuff
            if ('JSDBGP' in event.currentTarget) {
                if (this.param) {
                    var cmd = 'JSDBGP.winger_command("'+this.param+'", '+this.jsshell+');';
                    event.currentTarget.setTimeout(cmd,1000);
                    this.param = null;
                }
            }
        } catch(e) {
            dump(e+"\n");
        }
    }    
};

const jsdbgp_clh_contractID = "@activestate.com/jsdbgp/jsdbgp-clh;1";
const jsdbgp_clh_CID = Components.ID("{5be7db04-21ba-11db-bba7-000d935d3368}");
const jsdbgp_clh_category = "c-jsdbgp";

const JSDBGPConsoleHandlerModule = {
    /* nsISupports */

    QueryInterface : function(iid) {
        if (iid.equals(nsIModule) ||
            iid.equals(nsISupports))
            return this;

        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    /* nsIModule */

    getClassObject : function(compMgr, cid, iid) {
        if (cid.equals(jsdbgp_clh_CID))
            return JSDBGPConsoleHandler.QueryInterface(iid);

        throw Components.results.NS_ERROR_NOT_REGISTERED;
    },

    registerSelf : function(compMgr, fileSpec, location, type) {
        compMgr.QueryInterface(nsIComponentRegistrar);

        compMgr.registerFactoryLocation(jsdbgp_clh_CID,
                                        "JSDBGPConsoleHandler",
                                        jsdbgp_clh_contractID,
                                        fileSpec,
                                        location,
                                        type);

        var catMan = Components.classes["@mozilla.org/categorymanager;1"]
                               .getService(nsICategoryManager);
        catMan.addCategoryEntry("command-line-handler",
                                jsdbgp_clh_category,
                                jsdbgp_clh_contractID, true, true);
        
        try {
            var jsds = Components.classes["@mozilla.org/js/jsd/debugger-service;1"].getService(Components.interfaces.jsdIDebuggerService);
            jsds.initAtStartup = true;
        } catch(e) {
            dump(e+"\n");
        }
        
    },

    unregisterSelf : function(compMgr, location, type) {
        compMgr.QueryInterface(nsIComponentRegistrar);

        compMgr.unregisterFactoryLocation(jsdbgp_clh_CID, location);

        var catMan = Components.classes["@mozilla.org/categorymanager;1"]
                               .getService(nsICategoryManager);
        catMan.deleteCategoryEntry("command-line-handler", jsdbgp_clh_category);
    },

    canUnload : function (compMgr) {
        return true;
    }
};

/* module initialisation */
function NSGetModule(comMgr, fileSpec) {
    return JSDBGPConsoleHandlerModule;
}
