/* -*- Mode: JavaScript; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * Copyright (c) 2006, 2007 ActiveState Software Inc.
 * Refer to the license in the Komodo IDE installation
 * for the licensing information of this source code.
 *
 * winger-scripts.js -- manage the URL and script objects
 *
 * Ideas borrowed from Venkman, but we aren't using future breakpoints,
 * and we can have more than one script associated with a breakpoint at any
 * time.
 */

if (typeof(JSDBGP) == 'undefined') {
    var JSDBGP = {};
}

/* Classes:
 *
 * + ScriptManager: wrapper around each URL that manages a bag of and scripts
 * + ScriptWrapper : smart wrapper around jsdIScript
 */


JSDBGP.Scripts = function(mainObj) {
    this.mainObj = mainObj;
    this.log = JSDBGP.Logging.getLogger("winger.scripts");
    this.log.setLevel(JSDBGP.Logging.DEBUG);
    this.http2file = {};
    this.file2http = {};

    this.setUriMap = function(featureValue) {
        // JS doesn't have constrained split like in Perl, so we need indexOf
        var splitPoint = featureValue.indexOf("##");
        if (splitPoint < 0) {
            this.mainObj.console.command.error_code = JSDBGP.Constants.E_PARSE_ERROR;
            throw new Error("Invalid format for urimap: [" + featureValue + "]");
        }
        var http_uri = featureValue.substr(0, splitPoint);
        if (http_uri.length == 0) {
            return 0;
        }
        var local_path = featureValue.substr(splitPoint+ 2);
        if (local_path.length == 0) {
            return 0;
        }
        var file_uri = this.mainObj.utils.pathToURI(local_path);
        // Check for extra slashes added by pathToURI()
        if (file_uri.substr(-1) == "/" && http_uri.substr(-1) != '/') {
            file_uri = file_uri.substr(0, file_uri.length - 1);
        }
        // Assume there are no many-to-one links from
        // the web server to the local site.
        // If there are, last one wins.
        this.http2file[http_uri] = file_uri;
        this.file2http[file_uri] = http_uri;



        return 1;
    };

    this.getUriMap = function() {
        var s = "";
        for (var uri in this.http2file) {
            var fileName = this.http2file[uri];
            if (s.length > 0) s += " ";
            s += uri + "##" + fileName;
        }
        return s;
    };

    this.finalInit = function() {
        this.mainObj.settings.registerFunction('urimap', this, this.getUriMap, this.setUriMap);
    };

    this.finalShutdown = function() {
        this.mainObj.settings.deregisterFunction('urimap');
    };

    this._existsIfLocalPath = function(fname) {
        var nsIOService = (Components.classes["@mozilla.org/network/io-service;1"]
                           .getService(Components.interfaces.nsIIOService));
        var utils = new FileUtils();
        try {
            var scheme = nsIOService.extractScheme(fname);
            if (!scheme) {





            } else if (scheme != 'file') {
                // It's not a file URI
                return true;
            } else {
                var localName = utils.urlToPath(fname);
                fname = localName;
            }
        } catch(ex) {
            this.log.exception(ex);
        }
        try {
            return utils.exists(fname);
        } catch(ex) {
            this.log.exception(ex);
        }
        // Not a file-thing;
        return true;
    }
            

    this._checkPrefixMap = function(mapper, uri) {
        var longest_pair = [0, ""];
        for (var pfx in mapper) {
            if (pfx.length > longest_pair[0] && uri.indexOf(pfx) == 0) {
                var candidate = mapper[pfx] + uri.substr(pfx.length);
                if (this._existsIfLocalPath(candidate)) {
                    longest_pair = [pfx.length, mapper[pfx]];
                }
            }
        }
        return longest_pair[1] + uri.substr(longest_pair[0]);
    };

    // Used by call-stack reporter and breakpoint-lister

    this.mapHTTP_URI_to_FileURI = function(http_uri) {
        return this._checkPrefixMap(this.http2file, http_uri);
    }

    // Used by attempts to set breakpoints

    this.mapFileURI_to_HTTP_URI = function(file_uri) {
        return this._checkPrefixMap(this.file2http, file_uri);
    };

    this.currentPCMap = function(is_dbgp_breakpoint) {
        var actual_mode = (arguments.length >= 1
                           ? is_dbgp_breakpoint
                           : this.mainObj.settings.settings['pretty_print'][0]);
        return (actual_mode
                ? JSDBGP.Constants.PCMAP_PRETTYPRINT
                : JSDBGP.Constants.PCMAP_SOURCETEXT);
    }
};

/* The main classes here */

// Constructor

// precondition: url comes in canonicalized.

// ScriptManager : code for each URL comes in as a set of JSDIScripts,
// so wrap them in one of these.

JSDBGP.Scripts.ScriptManager = function(scriptObj, url) {
    this.scriptObj = scriptObj;
    this.mainObj = scriptObj.mainObj;
    this.log = scriptObj.log;



    // This url needs to be canonical
    this.url = url;
    this.scriptWrappers = {};
};

JSDBGP.Scripts.ScriptManager.prototype.onScriptCreated = function(jsdScript) {
    var sw = new JSDBGP.Scripts.ScriptWrapper(this.scriptObj, jsdScript);
    var tag = jsdScript.tag;
    this.scriptWrappers[tag] = sw;
    this.mainObj.console.scriptWrappers[tag] = sw;
    sw.onScriptCreated(this.url);
};

JSDBGP.Scripts.ScriptManager.prototype.onScriptInvalidated = function(jsdScript) {
    var tag = jsdScript.tag;
    if (tag in this.scriptWrappers) {
        this.scriptWrappers[tag].onScriptInvalidated();
        delete this.scriptWrappers[tag];
    } else {



    }
    delete this.mainObj.console.scriptWrappers[tag];
};

//****************************************************************

// ScriptWrapper class -- wrap a jsdScript object

JSDBGP.Scripts.ScriptWrapper = function(scriptObj, jsdScript) {
    this.mainObj = scriptObj.mainObj;
    this.log = scriptObj.log;
    this.jsdScript = jsdScript;
};

/* All existing breakpoints must be line-oriented, as the breakpoints
 * in pretty-print views are deleted when the script is deleted.
 */

JSDBGP.Scripts.ScriptWrapper.prototype.onScriptCreated = function(url) {
    // If the debugger isn't running now there isn't much we can do here.
    if (!this.mainObj.console.initialized) {
        return;
    }
    
    // First tie any existing breakpoints to this script if applicable
    var breakpointManager = this.mainObj.breakpointManager;
    var active_lines = this.mainObj.bpsMediator.getSourceBreakpoints(url);
    // active_lines: hash of line# => bkpt id
    if (!active_lines || this.mainObj.utils.keysLength(active_lines) == 0) {
        return;
    }

    var jsdScript = this.jsdScript;
    
    if (!jsdScript.isValid) {



        return;
    }
    var tag = jsdScript.tag;
    var startLine = jsdScript.baseLineNumber;
    var endLine = startLine + jsdScript.lineExtent;
    









































    var pcmap_mode = JSDBGP.Constants.PCMAP_SOURCETEXT;
    for (var line in active_lines) {
        if (startLine <= line && line <= endLine) {
            if (jsdScript.isLineExecutable(line, pcmap_mode)) {
                for (var bkpt_id in active_lines[line]) {
                    var bi = breakpointManager.getBreakpoint(bkpt_id);
                    if (bi) {
                        var pc = breakpointManager.bindSrcBreakpoint(bi, tag);
                    } else {



                    }
                }
            } else {
                // Don't panic if you get this message -- most likely we're
                // reading in a top-level script and don't want to set
                // breakpoints inside functions, only top-level code.
                



            }
        }
    }
};

JSDBGP.Scripts.ScriptWrapper.prototype.onScriptInvalidated = function() {
    this.mainObj.console.command.onScriptInvalidated(this.jsdScript.tag);
};
